# -*- coding: utf-8 -*-
"""Wavefront SDK Metric.

@author Hao Song (songhao@vmware.com)
"""


# pylint: disable=too-few-public-methods,E0012,R0205
class WavefrontSdkMetric(object):
    """Wavefront SDK Metric."""
